/*
ID: espr1t
TASK: Kites
KEYWORDS: Hard, DP, Geometry, Divide & Conquer
*/

#include <cstdio>
#include <cstring>
#include <algorithm>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 52;
const int MOD = 1000000007;

int n;
int a[MAX];
int b[MAX][2];
int dyn[MAX][MAX][MAX][MAX];

int farea(int x1, int y1, int x2, int y2, int x3, int y3) {
    return x1 * y2 + x2 * y3 + x3 * y1 - x1 * y3 - x2 * y1 - x3 * y2;
}

bool cross(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4) {
    if (farea(x1, y1, x2, y2, x3, y3) < 0 && farea(x1, y1, x2, y2, x4, y4) < 0)
        return false;
    if (farea(x3, y3, x4, y4, x1, y1) > 0 && farea(x3, y3, x4, y4, x2, y2) > 0)
        return false;
    return true;
}

int recurse(int left, int wleft, int right, int wright) {
    if (left + 1 >= right)
        return 1;
    if (dyn[left][wleft][right][wright] != -1)
        return dyn[left][wleft][right][wright];
    
    // Find highest kite in this region
    int idx = -1;
    for (int i = 1; i <= n; i++) {
        if (b[i][1] <= b[wleft][1] && b[i][1] <= b[wright][1]) {
            if (idx != -1 && b[i][1] <= b[idx][1])
                continue;
            if (cross(a[left], 0, b[wleft][0], b[wleft][1], a[left + 1], 0, b[i][0], b[i][1]))
                continue;
            if (cross(a[left + 1], 0, b[i][0], b[i][1], a[right], 0, b[wright][0], b[wright][1]))
                continue;
            idx = i;
        }
    }

    // Try each of the people, try the kite as his/hers
    int ans = 0;
    if (idx != -1) {
        for (int i = left + 1; i < right; i++) {
            int cur = 1;
            cur = ((long long)cur * recurse(left, wleft, i, idx)) % MOD;
            cur = ((long long)cur * recurse(i, idx, right, wright)) % MOD;
            ans += cur;
            if (ans >= MOD) ans -= MOD;
        }
    }
    
    return dyn[left][wleft][right][wright] = ans;
}

int main(void) {
    // in = fopen("Kites.in", "rt");
    
    fscanf(in, "%d", &n);
    for (int i = 1; i <= n; i++)
        fscanf(in, "%d", &a[i]);
    for (int i = 1; i <= n; i++)
        fscanf(in, "%d %d", &b[i][0], &b[i][1]);

    sort(a + 1, a + n + 1);

    // Set sentinels on both side of the points
    a[0] = 0, b[0][0] = 0, b[0][1] = 5005;
    a[n + 1] = 5005, b[n + 1][0] = 5005, b[n + 1][1] = 5005;
    
    memset(dyn, -1, sizeof(dyn));
    fprintf(out, "%d\n", recurse(0, 0, n + 1, n + 1));

    return 0;
}
